class FaradayMiddleware::Avro::EncodeAvro < Faraday::Middleware
  CONTENT_TYPE = 'Content-Type'.freeze
  MIME_TYPE    = 'avro/binary'.freeze

  def initialize(app, options={})
    super(app)
    @schemas_path = \
      options[:schemas_path] || File.expand_path(File.dirname(__FILE__))
  end

  def call(env)
    match_content_type(env) do |data|
      schema = env[:request_headers]['Avro-Schema']
      env[:body] = encode(data, schema)
    end

    @app.call env
  end

  def encode(data, schema)
    avro = AvroTurf.new(schemas_path: @schemas_path)
    data = JSON.parse data if data.respond_to?(:to_str)

    avro.encode(data, schema_name: schema)
  end

  def match_content_type(env)
    if process_request?(env)
      env[:request_headers][CONTENT_TYPE] ||= MIME_TYPE
      yield env[:body]
    end
  end

  def process_request?(env)
    type = request_type(env)
    has_body?(env) and (type.empty? or type == MIME_TYPE)
  end

  def has_body?(env)
    body = env[:body] and !(body.respond_to?(:to_str) and body.empty?)
  end

  def request_type(env)
    type = env[:request_headers][CONTENT_TYPE].to_s
    type = type.split(';', 2).first if type.index(';')
    type
  end
end
